@@ -1,6 +1,12 @@
 """skimpy provides summary statistics about variables in pandas data frames."""
+import re
 from collections import defaultdict
 from itertools import chain
+from typing import Any
+from typing import Dict
+from typing import List
+from typing import Optional
+from unicodedata import normalize
 
 import numpy as np
 import pandas as pd
@@ -14,6 +20,20 @@ from rich.table import Table
 from rich.text import Text
 from typeguard import typechecked
 
+NULL_VALUES = {np.nan, "", None}
+
+CASE_STYLES = {
+    "snake",
+    "kebab",
+    "camel",
+    "pascal",
+    "const",
+    "sentence",
+    "title",
+    "lower",
+    "upper",
+}
+
 console = Console()
 QUANTILES = [0, 0.25, 0.75, 1]
 HIST_BINS = 6
@@ -28,6 +48,10 @@ UNICODE_HIST = {
     7 / 8: "▇",
     1: "█",
 }
+# These are defined globally because they are used in more than one function
+DATE_COL_FIRST = "first"
+DATE_COL_LAST = "last"
+NUM_COL_MEAN = "mean"
 
 
 @typechecked
@@ -46,26 +70,21 @@ def infer_datatypes(df: pd.DataFrame) -> pd.DataFrame:
     )
     loop_types = df_types.values.tolist()
     for col in loop_types:
-        if col[1] == "mixed":
-            pass
+        if col[1] == "string":
+            data_type = "string"
+        elif col[1] == "integer":
+            data_type = "int"
+        elif col[1] == "floating":
+            data_type = "float64"
+        elif col[1] == "datetime64":
+            data_type = "datetime64"
+        elif col[1] == "categorical":
+            data_type = "category"
+        elif col[1] == "boolean":
+            data_type = "bool"
         else:
-            if col[1] == "decimal":
-                data_type = "float64"
-            elif col[1] == "string":
-                data_type = "string"
-            elif col[1] == "integer":
-                data_type = "int"
-            elif col[1] == "floating":
-                data_type = "float64"
-            elif col[1] == "date":
-                data_type = "datetime64"
-            elif col[1] == "categorical":
-                data_type = "category"
-            elif col[1] == "boolean":
-                data_type = "bool"
-            else:
-                data_type = col[1]
-            df[col[0]] = df[col[0]].astype(data_type)
+            data_type = col[1]
+        df[col[0]] = df[col[0]].astype(data_type)
     return df
 
 
@@ -84,6 +103,69 @@ def round_dataframe(df: pd.DataFrame) -> pd.DataFrame:
     return df
 
 
+@typechecked
+def map_row_positions_to_text_style(types_to_property: dict, df: pd.DataFrame) -> dict:
+    """Maps positions in summary dataframe (eg row) to a Rich text property.
+
+    :param types_to_property: Datatype, datetime, mapping to Rich text property
+    :type types_to_property: dict
+    :param df: Dataframe to map positions in
+    :type df: pd.DataFrame
+    :return: Dictionary mapping row position to Rich text property
+    :rtype: dict
+    """
+    cols_to_cat_map = dict(
+        zip(
+            types_to_property.keys(),
+            [
+                list(df.select_dtypes(entry).columns)
+                for entry in types_to_property.keys()
+            ],
+        )
+    )
+    type_by_col_default = defaultdict(list)
+    for k, seq in cols_to_cat_map.items():
+        for letter in seq:
+            type_by_col_default[letter].append(k)
+    type_by_col = dict(
+        zip(type_by_col_default.keys(), [x[0] for x in type_by_col_default.values()])
+    )
+    type_to_prop = dict(
+        zip(type_by_col.keys(), [types_to_property[x] for x in type_by_col.values()])
+    )
+    columns = list(df.columns)
+    row_pos_to_property = dict(
+        zip(range(len(columns)), [type_to_prop[x] for x in columns])
+    )
+    return row_pos_to_property
+
+
+@typechecked
+def simplify_datetimes_in_array(rows: np.ndarray) -> np.ndarray:
+    """Simplifies 2001/01/01 00:00:00 to 2001/01/01.
+
+    :param rows: contain summary info, including datetimes
+    :type rows: np.ndarray
+    :return: rows with any all zero hours/min/sec stripped out
+    :rtype: np.ndarray
+    """
+    timestamp_positions = [
+        [
+            [idx, i]
+            for i, j in enumerate(item)
+            if type(j) == pd._libs.tslibs.timestamps.Timestamp
+        ]
+        for idx, item in enumerate(rows)
+    ]
+    timestamp_pos_list = list(chain.from_iterable(timestamp_positions))
+    timestamp_pos_tuples = [tuple(entry) for entry in timestamp_pos_list]
+    for entry in timestamp_pos_tuples:
+        hour, min, sec = rows[entry].hour, rows[entry].minute, rows[entry].second
+        if hour == min == sec == 0:
+            rows[entry] = rows[entry].strftime("%Y-%m-%d")
+    return rows
+
+
 @typechecked
 def dataframe_to_rich_table(
     table_name: str,
@@ -130,51 +212,37 @@ def dataframe_to_rich_table(
         "bool": bool,
         "object": object,
     }
-    cols_to_cat_map = dict(
-        zip(
-            datatype_colours.keys(),
-            [
-                list(df.select_dtypes(entry).columns)
-                for entry in datatype_colours.keys()
-            ],
-        )
-    )
-    type_by_col_default = defaultdict(list)
-    for k, seq in cols_to_cat_map.items():
-        for letter in seq:
-            type_by_col_default[letter].append(k)
-    type_by_col = dict(
-        zip(type_by_col_default.keys(), [x[0] for x in type_by_col_default.values()])
-    )
-    type_to_colour = dict(
-        zip(type_by_col.keys(), [datatype_colours[x] for x in type_by_col.values()])
-    )
-    columns = list(df.columns)
-    col_pos_to_colour = dict(
-        zip(range(len(columns)), [type_to_colour[x] for x in columns])
-    )
+    # generate dict of types to text justifications
+    datatype_justify = {
+        "number": "right",
+        "category": "center",
+        "datetime": "center",
+        "string": "center",
+        "bool": "left",
+        "object": "left",
+    }
+    pos_to_colour = map_row_positions_to_text_style(datatype_colours, df)
+    pos_to_justification = map_row_positions_to_text_style(datatype_justify, df)
     rows = df.values
     # find any datetimes
-    if ("first" or "last") in df.columns:
-        timestamp_positions = [
-            [
-                [idx, i]
-                for i, j in enumerate(item)
-                if type(j) == pd._libs.tslibs.timestamps.Timestamp
-            ]
-            for idx, item in enumerate(rows)
+    if (DATE_COL_FIRST or DATE_COL_LAST) in df.columns:
+        rows = simplify_datetimes_in_array(rows)
+    rows = [
+        [
+            str(s).rstrip("0").rstrip(".") if ("." and type(s) == float) else s
+            for s in row
         ]
-        timestamp_pos_list = list(chain.from_iterable(timestamp_positions))
-        timestamp_pos_tuples = [tuple(entry) for entry in timestamp_pos_list]
-        for entry in timestamp_pos_tuples:
-            hour, min, sec = rows[entry].hour, rows[entry].minute, rows[entry].second
-            if hour == min == sec == 0:
-                rows[entry] = rows[entry].strftime("%Y-%m-%d")
-    for col in columns:
+        for row in rows
+    ]
+    for col in df.columns:
         table.add_column(str(col), overflow="fold")
     for row in rows:
         row = [
-            Text(str(item)[:str_limit], style=col_pos_to_colour[i])
+            Text(
+                str(item)[:str_limit],
+                style=pos_to_colour[i],
+                justify=pos_to_justification[i],
+            )
             for i, item in enumerate(row)
         ]
         table.add_row(*list(row))
@@ -238,7 +306,7 @@ def numeric_variable_summary_table(xf: pd.DataFrame) -> pd.DataFrame:
     data_dict = {
         "missing": count_nans_vec,
         "complete rate": 1 - count_nans_vec / xf.shape[0],
-        "mean": xf.mean(),
+        NUM_COL_MEAN: xf.mean(),
         "sd": xf.std(),
     }
     display_quantiles_as_pct = 100
@@ -365,7 +433,7 @@ def datetime_variable_summary_table(xf: pd.DataFrame) -> pd.DataFrame:
     data_dict = {
         "missing": count_nans_vec,
         "complete rate": 1 - count_nans_vec / xf.shape[0],
-        "first": pd.Series(
+        DATE_COL_FIRST: pd.Series(
             dict(
                 zip(
                     xf.columns,
@@ -373,7 +441,9 @@ def datetime_variable_summary_table(xf: pd.DataFrame) -> pd.DataFrame:
                 )
             )
         ),
-        "last": pd.Series(dict(zip(xf.columns, [xf[col].max() for col in xf.columns]))),
+        DATE_COL_LAST: pd.Series(
+            dict(zip(xf.columns, [xf[col].max() for col in xf.columns]))
+        ),
     }
     if len(xf) > 3:
         data_dict.update(
@@ -459,7 +529,7 @@ def skim(
             sum_df = summary_func(xf)
             list_of_tabs.append(
                 dataframe_to_rich_table(
-                    col_type, round_dataframe(sum_df), **colour_kwargs
+                    col_type, round_dataframe(sum_df)  # , **colour_kwargs
                 )
             )
     # Put all of the info together
@@ -476,6 +546,198 @@ def skim(
     console.print(Panel(grid, title="skimpy summary", subtitle="End"))
 
 
+@typechecked
+def clean_columns(
+    df: pd.DataFrame,
+    case: str = "snake",
+    replace: Optional[Dict[str, str]] = None,
+    remove_accents: bool = True,
+) -> pd.DataFrame:
+    """Function to clean column names, originally from the dataprep python package.
+
+    Parameters
+    ----------
+    df
+        Dataframe from which column names are to be cleaned.
+    case
+        The desired case style of the column name.
+            - 'snake': 'column_name'
+            - 'kebab': 'column-name'
+            - 'camel': 'columnName'
+            - 'pascal': 'ColumnName'
+            - 'const': 'COLUMN_NAME'
+            - 'sentence': 'Column name'
+            - 'title': 'Column Name'
+            - 'lower': 'column name'
+            - 'upper': 'COLUMN NAME'
+        (default: 'snake')
+    replace
+        Values to replace in the column names.
+            - {'old_value': 'new_value'}
+        (default: None)
+    remove_accents
+        If True, strip accents from the column names.
+        (default: True)
+    Examples
+    --------
+    Clean column names by converting the names to camel case style, removing accents,
+    and correcting a mispelling.
+    >>> df = pd.DataFrame({'FirstNom': ['Philip', 'Turanga'], 'lastName': ['Fry', 'Leela'], \
+'Téléphone': ['555-234-5678', '(604) 111-2335']})
+    >>> clean_headers(df, case='camel', replace={'Nom': 'Name'})
+    Column Headers Cleaning Report:
+        2 values cleaned (66.67%)
+      firstName lastName       telephone
+    0    Philip      Fry    555-234-5678
+    1   Turanga    Leela  (604) 111-2335
+    """
+    if case not in CASE_STYLES:
+        raise ValueError(
+            f"case {case} is invalid, it needs to be one of {', '.join(c for c in CASE_STYLES)}"
+        )
+
+    # Store original column names for creating cleaning report
+    orig_columns = df.columns.astype(str).tolist()
+
+    if replace:
+        df = df.rename(columns=lambda col: _replace_values(col, replace))
+
+    if remove_accents:
+        df = df.rename(columns=_remove_accents)
+
+    df = df.rename(columns=lambda col: _convert_case(col, case))
+    df.columns = _rename_duplicates(df.columns, case)
+    # Count the number of changed column names
+    new_columns = df.columns.astype(str).tolist()
+    cleaned = [
+        1 if new_columns[i] != orig_columns[i] else 0 for i in range(len(orig_columns))
+    ]
+    stats = {"cleaned": sum(cleaned)}
+
+    return df
+
+
+@typechecked
+def _convert_case(name: Any, case: str) -> Any:
+    """Convert case style of a column name.
+
+    Parameters
+    ----------
+    name
+        Column name.
+    case
+        The desired case style of the column name.
+    """
+    if name in NULL_VALUES:
+        name = "header"
+
+    if case in {"snake", "kebab", "camel", "pascal", "const"}:
+        words = _split_strip_string(str(name))
+    else:
+        words = _split_string(str(name))
+
+    if case == "snake":
+        name = "_".join(words).lower()
+    elif case == "kebab":
+        name = "-".join(words).lower()
+    elif case == "camel":
+        name = words[0].lower() + "".join(w.capitalize() for w in words[1:])
+    elif case == "pascal":
+        name = "".join(w.capitalize() for w in words)
+    elif case == "const":
+        name = "_".join(words).upper()
+    elif case == "sentence":
+        name = " ".join(words).capitalize()
+    elif case == "title":
+        name = " ".join(w.capitalize() for w in words)
+    elif case == "lower":
+        name = " ".join(words).lower()
+    elif case == "upper":
+        name = " ".join(words).upper()
+
+    return name
+
+
+@typechecked
+def _split_strip_string(string: str) -> List[str]:
+    """Split the string into separate words and strip punctuation."""
+    string = re.sub(r"[!()*+\,\-./:;<=>?[\]^_{|}~]", " ", string)
+    string = re.sub(r"[\'\"\`]", "", string)
+
+    return re.sub(
+        r"([A-Z][a-z]+)", r" \1", re.sub(r"([A-Z]+|[0-9]+|\W+)", r" \1", string)
+    ).split()
+
+
+@typechecked
+def _split_string(string: str) -> List[str]:
+    """Split the string into separate words."""
+    string = re.sub(r"[\-_]", " ", string)
+
+    return re.sub(r"([A-Z][a-z]+)", r" \1", re.sub(r"([A-Z]+)", r"\1", string)).split()
+
+
+@typechecked
+def _replace_values(name: Any, mapping: Dict[str, str]) -> Any:
+    """Replace string values in the column name.
+
+    Parameters
+    ----------
+    name
+        Column name.
+    mapping
+        Maps old values in the column name to the new values.
+    """
+    if name in NULL_VALUES:
+        return name
+
+    name = str(name)
+    for old_value, new_value in mapping.items():
+        # If the old value or the new value is not alphanumeric, add underscores to the
+        # beginning and end so the new value will be parsed correctly for _convert_case()
+        new_val = (
+            fr"{new_value}"
+            if old_value.isalnum() and new_value.isalnum()
+            else fr"_{new_value}_"
+        )
+        name = re.sub(fr"{old_value}", new_val, name, flags=re.IGNORECASE)
+
+    return name
+
+
+@typechecked
+def _remove_accents(name: Any) -> Any:
+    """Return the normal form for a Unicode string name using canonical decomposition."""
+    if not isinstance(name, str):
+        return name
+
+    return normalize("NFD", name).encode("ascii", "ignore").decode("ascii")
+
+
+@typechecked
+def _rename_duplicates(names: pd.Index, case: str) -> Any:
+    """Rename duplicated column names to append a number at the end."""
+    if case in {"snake", "const"}:
+        sep = "_"
+    elif case in {"camel", "pascal"}:
+        sep = ""
+    elif case == "kebab":
+        sep = "-"
+    else:
+        sep = " "
+
+    names = list(names)
+    counts: Dict[str, int] = {}
+
+    for i, col in enumerate(names):
+        cur_count = counts.get(col, 0)
+        if cur_count > 0:
+            names[i] = f"{col}{sep}{cur_count}"
+        counts[col] = cur_count + 1
+
+    return names
+
+
 @typechecked
 def generate_test_data() -> pd.DataFrame:
     """Generate dataframe with several different datatypes.
